/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphItem } from "../SGraphItem";
import { SPainter, SRect } from "@persagy-web/draw";
import { SLineStyle } from "..";
import { SPoint } from "@persagy-web/draw/lib";
import { SGraphStyleItem } from "./SGraphStyleItem";

/**
 * 矩形(包含圆角矩形) item
 *
 * @author 郝洁 <haojie@persagy.com>
 */
export class SGraphRectItem extends SGraphStyleItem {
    /** 矩形两个对角 */
    private _line: SPoint[] = [];
    get line(): SPoint[] {
        return this._line;
    }
    set line(arr: SPoint[]) {
        this._line = arr;
        this.update();
    }

    /** 矩形左上角 */
    protected _leftTop: SPoint = new SPoint();
    /** 矩形右下角 */
    protected _rightBottom: SPoint = new SPoint();

    /** 绘制矩形的圆角半径 */
    private _radius: number = 0;
    get radius(): number {
        return this._radius;
    }
    set radius(v: number) {
        if (v == this._radius) {
            return;
        }
        this._radius = v;
        this.update();
    }

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      矩形数据
     * @param radius    圆角半径
     */
    constructor(parent: SGraphItem | null, data: SPoint[], radius: number = 0) {
        super(parent);
        if (data.length) {
            this.line = data;
            this.calRect();
        }
        this.radius = radius;
    }

    /**
     * Item 对象边界区域
     *
     * @return	边界区域
     */
    boundingRect(): SRect {
        if (this.line.length > 1) {
            return new SRect(this._leftTop, this._rightBottom);
        }

        return new SRect();
    }

    /**
     * 计算矩形的左上角和右下角
     */
    protected calRect(): void {
        if (this.line.length > 1) {
            const fi = this.line[0];
            const se = this.line[1];
            let minx, maxx, miny, maxy;
            if (fi.x < se.x) {
                minx = fi.x;
                maxx = se.x;
            } else {
                minx = se.x;
                maxx = fi.x;
            }

            if (fi.y < se.y) {
                miny = fi.y;
                maxy = se.y;
            } else {
                miny = se.y;
                maxy = fi.y;
            }

            this._leftTop = new SPoint(minx, miny);
            this._rightBottom = new SPoint(maxx, maxy);
        }
    } // Function calRect()

    /**
     * 绘制
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        if (this.line.length == 2) {
            painter.pen.color = this.strokeColor;
            painter.brush.color = this.fillColor;
            if (this.widthIsPx) {
                painter.pen.lineWidth = painter.toPx(this.lineWidth);
            } else {
                painter.pen.lineWidth = this.lineWidth;
            }
            if (this.lineStyle == SLineStyle.Dashed) {
                painter.pen.lineDash = [
                    painter.toPx(this.lineWidth * 3),
                    painter.toPx(this.lineWidth * 7)
                ];
            } else if (this.lineStyle == SLineStyle.Dotted) {
                painter.pen.lineDash = [
                    painter.toPx(this.lineWidth * 2),
                    painter.toPx(this.lineWidth * 2)
                ];
            }

            if (this.radius != 0) {
                painter.drawRoundRect(
                    this._leftTop,
                    this._rightBottom,
                    this.radius
                );
            } else {
                painter.drawRect(this._leftTop, this._rightBottom);
            }
        }
    }
}
