/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphItem } from "../SGraphItem";
import {
    SColor,
    SGradient,
    SLinearGradient,
    SRadialGradient
} from "@persagy-web/draw/lib";
import { SLineStyle } from "../enums/SGraphLineStyle";

/**
 * 样式 item
 *
 * @author 郝建龙 <haojianlong@sagacloud.cn>
 */
export class SGraphStyleItem extends SGraphItem {
    /** 笔触颜色 */
    _strokeColor: SColor = SColor.Black;
    get strokeColor(): SColor {
        return this._strokeColor;
    } // Get strokeColor
    set strokeColor(v: SColor) {
        this._strokeColor = new SColor(v);
        this.update();
    } // Set strokeColor

    /** 填充颜色 */
    _fillColor: SColor = SColor.Black;
    get fillColor(): SColor {
        return this._fillColor;
    } // Get fillColor
    set fillColor(v: SColor) {
        this._fillColor = new SColor(v);
        this.update();
    } // Set fillColor

    /** 线条样式 */
    _lineStyle: SLineStyle = SLineStyle.Solid;
    get lineStyle(): SLineStyle {
        return this._lineStyle;
    } // Get lineStyle
    set lineStyle(v: SLineStyle) {
        this._lineStyle = v;
        this.update();
    } // Set lineStyle

    /** 线条宽度 */
    _lineWidth: number = 1;
    get lineWidth(): number {
        return this._lineWidth;
    } // Get lineWidth
    set lineWidth(v: number) {
        this._lineWidth = v;
        this.update();
    } // Set lineWidth

    /** 宽度是否像素 */
    widthIsPx: boolean = false;

    /**
     * 构造函数
     *
     * @param parent    父级
     */
    constructor(parent: SGraphItem | null = null) {
        super(parent);
    }

    /**
     * 解析渐变样式
     *
     * @param str   渐变存储的字符串
     * @return 转化为的渐变
     */
    parseFill(str: string): SGradient | undefined {
        const index1 = str.indexOf("{");
        const index2 = str.indexOf("}");
        const temp = str.substring(index1 + 1, index2 - 1);
        const dataArr = temp.split(";");
        if (str.indexOf("SLinearGradient") > -1) {
            try {
                // @ts-ignore
                const grad = new SLinearGradient(
                    ...dataArr[0].split(","),
                    // @ts-ignore
                    ...dataArr[1].split(",")
                );
                for (let i = 2; i < dataArr.length; i++) {
                    const cur = dataArr[i].split(",");
                    grad.addColorStop(Number(cur[0]), new SColor(cur[1]));
                }
                return grad;
            } catch (e) {
                console.log("渐变颜色格式有误", e, str);
            }
        } else if (str.indexOf("SRadialGradient") > -1) {
            try {
                // @ts-ignore
                const grad = new SRadialGradient(
                    ...dataArr[0].split(","),
                    // @ts-ignore
                    ...dataArr[1].split(",")
                );
                for (let i = 2; i < dataArr.length; i++) {
                    const cur = dataArr[i].split(",");
                    grad.addColorStop(Number(cur[0]), new SColor(cur[1]));
                }
                return grad;
            } catch (e) {
                console.log("渐变颜色格式有误", e, str);
            }
        }
        return undefined;
    } // parseFill()
}
